local fs = require "nixio.fs" 

m = Map("iec104-master-slave", translate("IEC-104 Master Slave"))

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
	translate("Enable IEC-104 Master Slave"))
option_enabled.rmempty=false
option_enabled.default="1"

s = m:section(TypedSection, "configuration_master", "IEC 104 Master configuration")
s.anonymous = true


-- iec104 master configuration --

host = s:option(Value, "host", "Host", translate("IEC60870 server's hostname or IP"))
host.datatype = "ip4addr"
host.default = "127.0.0.1"

port = s:option(Value, "port", "Port", translate("IEC60870 server's port number"))
port.datatype = "port"
port.default = "2404"

k = s:option(Value, "k", "K", translate("Maximum unacknowledged transmitted APDUs"))
k.datatype = "integer"
k.default = "12"

w = s:option(Value, "w", "W", translate("Maximum unacknowledged received APDUs"))
w.datatype = "integer"
w.default = "8"

t0 = s:option(Value, "t0", "T0", translate("Amount of time that the IEC60870 Master will attempt to connect to the device"))
t0.datatype = "range(1,30)"
t0.default = "2"

t1 = s:option(Value, "t1", "T1", translate("Time to wait for a acknowledgment (“ACK”) to a transmitted APDU"))
t1.datatype = "integer"
t1.default = "15"

t2 = s:option(Value, "t2", "T2", translate("Time to wait before sending a supervisory APDU ACK"))
t2.datatype = "integer"
t2.default = "10"

t3 = s:option(Value, "t3", "T3", translate("Idle time before sending TEST APDU"))
t3.datatype = "integer"
t3.default = "20"

sizeof_cot = s:option(ListValue, "sizeof_cot", "Size Of COT", translate("Specify the number of octets in an ASDU COT field"))
sizeof_cot:value("1", translate("1"))
sizeof_cot:value("2", translate("2"))
sizeof_cot.default = "2"

oa = s:option(Value, "oa", "OA", translate("This is required when there is more than one controlled station(master/client)"))
oa.datatype = "range(0,254)"
oa.default = "0"

ca = s:option(Value, "ca", "CA", translate("Common Address or ASDU Address"))
ca.datatype = "range(0,254)"
ca.default = "1"

sizeof_ca = s:option(ListValue, "sizeof_ca", "Size Of CA", translate("Size Of Common Address or ASDU Address for IEC101 is 1 and for  IEC104 is 2"))
sizeof_ca:value("1", translate("1"))
sizeof_ca:value("2", translate("2"))
sizeof_ca.default = "2"

sizeof_ioa = s:option(ListValue, "sizeof_ioa", "Size Of IOA", translate("Size Of Information Object Address"))
sizeof_ioa:value("1", translate("1"))
sizeof_ioa:value("2", translate("2"))
sizeof_ioa:value("3", translate("3"))
sizeof_ioa.default = "3"

max_asdu_length = s:option(ListValue, "max_asdu_length", "Max. ASDU length", translate("Maximum size of ASDU that is generated,Its value is 249 for IEC104 and 254 for IEC101."))
max_asdu_length:value("249", translate("249"))
max_asdu_length:value("254", translate("254"))
max_asdu_length.default = "249"


-- iec104 slave configuration --

s2 = m:section(TypedSection,"configuration_slave",translate("IEC 104 Slave Configuration"))
s2.anonymous =true

no_of_masters = s2:option(Value, "no_of_masters", "Number Of Master Connections", translate("Maximum number of simultaneous IEC104-Master connections"))
no_of_masters.datatype = "integer"
no_of_masters.default = "1"

k_slave = s2:option(Value, "k", "K", translate("Maximum unacknowledged transmitted APDUs"))
k_slave.datatype = "integer"
k_slave.default = "12"

w_slave = s2:option(Value, "w", "W", translate("Maximum unacknowledged received APDUs"))
w_slave.datatype = "integer"
w_slave.default = "8"

t0_slave = s2:option(Value, "t0", "T0", translate("Amount of time that the IEC60870 Master will attempt to connect to the device"))
t0_slave.datatype = "range(1,30)"
t0_slave.default = "2"

t1_slave = s2:option(Value, "t1", "T1", translate("Time to wait for a acknowledgment (“ACK”) to a transmitted APDU"))
t1_slave.datatype = "integer"
t1_slave.default = "15"

t2_slave = s2:option(Value, "t2", "T2", translate("Time to wait before sending a supervisory APDU ACK"))
t2_slave.datatype = "integer"
t2_slave.default = "10"

t3_slave = s2:option(Value, "t3", "T3", translate("Idle time before sending TEST APDU"))
t3_slave.datatype = "integer"
t3_slave.default = "20"

sizeof_cot_slave = s2:option(ListValue, "sizeof_cot", "Size Of COT", translate("Specify the number of octets in an ASDU COT field"))
sizeof_cot_slave:value("1", translate("1"))
sizeof_cot_slave:value("2", translate("2"))
sizeof_cot_slave.default = "2"

oa_slave = s2:option(Value, "oa", "OA", translate("This is required when there is more than one controlled station(master/client)"))
oa_slave.datatype = "range(0,254)"
oa_slave.default = "0"

ca_slave = s2:option(Value, "ca", "CA", translate("Common Address or ASDU Address"))
ca_slave.datatype = "range(0,254)"
ca_slave.default = "1"

sizeof_ca_slave = s2:option(ListValue, "sizeof_ca", "Size Of CA", translate("Size Of Common Address or ASDU Address for IEC101 is 1 and for  IEC104 is 2"))
sizeof_ca_slave:value("1", translate("1"))
sizeof_ca_slave:value("2", translate("2"))
sizeof_ca_slave.default = "2"

sizeof_ioa_slave = s2:option(ListValue, "sizeof_ioa", "Size Of IOA", translate("Size Of Information Object Address"))
sizeof_ioa_slave:value("1", translate("1"))
sizeof_ioa_slave:value("2", translate("2"))
sizeof_ioa_slave:value("3", translate("3"))
sizeof_ioa_slave.default = "3"

max_asdu_length_slave = s2:option(ListValue, "max_asdu_length", "Max. ASDU length", translate("Maximum size of ASDU that is generated,Its value is 249 for IEC104 and 254 for IEC101."))
max_asdu_length_slave:value("249", translate("249"))
max_asdu_length_slave:value("254", translate("254"))
max_asdu_length_slave.default = "249"


return m

